@@ -6,7 +6,7 @@ use Carp qw/croak/;
 use POSIX qw/ceil/;
 
 
-our $VERSION = '1.008';
+our $VERSION = '1.010';
 
 require XSLoader;
 XSLoader::load('Session::Token', $VERSION);
@@ -179,14 +179,14 @@ Generally speaking the generator should be kept around and re-used. Probably the
 
 However, re-using a generator may be undesirable because servers are typically started immediately after a system reboot and the kernel's randomness pool might be poorly seeded at that point meaning that all subsequently generated tokens may be based on a weak/predictable seed. For this reason, you might choose to defer creating the generator until the first request actually comes in and/or periodically re-create the generator object. 
 
-Aside: Some crappy (usually C) programs that assume opening C</dev/urandom> will always succeed can return session tokens based only on the contents of nulled or uninitialised memory (unix really ought to provide a system call for random data). Session::Token throws an exception if it can't seed itself.
+Aside: Some crappy (usually C) programs that assume opening C</dev/urandom> will always succeed can return session tokens based only on the contents of nulled or uninitialised memory (unix really ought to provide a system call for random data). The Session::Token constructor throws an exception if it can't seed itself.
 
 
 =head1 CUSTOM ALPHABETS
 
 Being able to choose exactly which characters appear in your token is sometimes useful. This set of characters is called the I<alphabet>. B<The default alphabet size is 62 characters: uppercase letters, lowercase letters, and digits> (C<a-zA-Z0-9>).
 
-For some purposes, base-62 is a sweet spot. It is more compact than hexadecimal encoding which helps with efficiency because session tokens are usually transfered over the network many times during a session (often uncompressed in HTTP headers).
+For some purposes, base-62 is a sweet spot. It is more compact than hexadecimal encoding which helps with efficiency because session tokens are usually transferred over the network many times during a session (often uncompressed in HTTP headers).
 
 Also, base-62 tokens don't use "wacky" characters like base-64 encodings do. These characters sometimes cause encoding/escaping problems (ie when embedded in URLs) and are annoying because often you can't select tokens by double-clicking on them.
 
@@ -303,7 +303,7 @@ This minor inefficiency isn't an issue because the ISAAC implementation used is
 
 =head1 INTRODUCING BIAS
 
-If your alphabet contains the same character two or more times, this character will be more biased than a character that only occurs once. You should be careful that your alphabets don't overlap if you are trying to create random session tokens.
+If your alphabet contains the same character two or more times, this character will be more biased than a character that only occurs once. You should be careful that your alphabets don't repeat in this way if you are trying to create random session tokens.
 
 However, if you wish to introduce bias this library doesn't try to stop you. (Maybe it should print a warning?)
 
@@ -335,6 +335,52 @@ This module is not however designed to be the ultimate random number generator a
 
 
 
+=head1 TOKEN TEMPLATES
+
+L<String::Random> has a method called C<randpattern> where you provide a pattern that serves as a template when creating the token. You define the meaning of 1 or more template characters and each one that occurs in the pattern is replaced by a random character from a corresponding alphabet.
+
+Andrew Beverley requested this feature for L<Session::Token> and I suggested approximately the following:
+
+    use Session::Token;
+
+    sub token_template {
+      my (%m) = @_;
+
+      %m = map { $_ => Session::Token->new(alphabet => $m{$_}, length => 1) } keys %m;
+
+      return sub {
+        my $v = shift;
+        $v =~ s/(.)/exists $m{$1} ? $m{$1}->get : $1/eg;
+        return $v;
+      };
+    }
+
+In order to use C<token_template> you should pass it key-vaue pairs of the different token characters and the alphabets they represent. It will return a sub that should be passed the template pattern and it will return the resulting random tokens.
+
+For example, here is how to create L<UUID version 4|https://en.wikipedia.org/wiki/Universally_unique_identifier#Version_4_.28random.29> tokens:
+
+    sub uuid_v4_generator {
+      my $t = token_template(
+            x => [ 0..9, 'a'..'f' ],
+            y => [ 8, 9, 'a', 'b' ],
+          );
+
+      return sub {
+        return $t->('xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx');
+      }
+    }
+
+C<uuid_v4_generator> returns a generator function that will return tokens of the following form:
+
+    1b782499-9913-4726-a80a-25e7b2221a7c
+    90f85a64-d826-43bf-98e7-94ba87406bfb
+    b8b73175-3cce-4861-b43b-3dec5ed5d641
+    3afb64ab-6de3-4647-bbff-eb94dfa7d4b0
+    447d2001-2aec-4d32-9910-8c289ae34c48
+
+Note that characters in the pattern which don't have template characters defined (C<-> and C<4> in the above example) are passed through to the output token.
+
+
 
 =head1 SEEDING
 
@@ -377,10 +423,29 @@ Doesn't work on perl 5.6 and below due to the use of C<:raw> (thanks CPAN tester
 On windows we use L<Crypt::Random::Source::Strong::Win32> which has a big dependency tree. We should instead use a slimmer module like L<Crypt::Random::Seed>.
 
 
+=head1 COMMAND-LINE APP
+
+There is a command-line application called L<App::Session::Token> which is a convenience wrapper around L<Session::Token>. You can generate session tokens by running the C<session-token> binary:
+
+    $ echo "Your password is `session-token`"
+    Your password is 8Yom6z4AeB1RXxCGzklJFt
+
+It supports all the options of this module via command line parameters, and multiple session tokens can be generated with the C<--num> (aka C<-n>) switch. For example:
+
+    $ session-token --alphabet ABC --entropy 32 --num 5
+    BACAACABCCCCAACBBBCAB
+    BCBACACBBCACCBABABCBA
+    ABBBCBABBACBBBCBBBCCA
+    AACCBBBCCAAACBABACABC
+    CCABCABBCCCAACAAACCAA
+
+
 =head1 SEE ALSO
 
 L<The Session::Token github repo|https://github.com/hoytech/Session-Token>
 
+L<App::Session::Token>
+
 There are lots of different modules for generating random data. If the characterisations of any of them below are inaccurate or out-of-date, please file a github issue and I will correct them.
 
 Like this module, perl's C<rand()> function implements a user-space PRNG seeded from C</dev/urandom>. However, perl's C<rand()> is not secure. Perl doesn't specify a PRNG algorithm at all. On linux, whatever it is is seeded with a mere 4 bytes from C</dev/urandom>.
@@ -389,16 +454,18 @@ L<Data::Token> is the first thing I saw when I looked around on CPAN. It has an
 
 There are several decent random number generators like L<Math::Random::Secure> and L<Crypt::URandom> but they usually don't implement alphabets and some of them require you open or read from C</dev/urandom> for every chunk of random bytes. Note that Math::Random::Secure does prevent mod bias in its random integers and could be used to implement unbiased alphabets (slowly).
 
-L<String::Random> has a neat regexp-like language for specifying random tokens which is more flexible than alphabets. However, it uses perl's C<rand()> and its documentation fails to discuss performance, bias, or security.
+L<String::Random> has a neat regexp-like language for specifying random tokens which is more flexible than alphabets. However, it uses perl's C<rand()> and its documentation fails to discuss performance, bias, or security. See the L</"TOKEN TEMPLATES"> section for a similar feature.
 
 L<String::Urandom> has alphabets, but it uses the flawed mod algorithm described above and opens C</dev/urandom> for every token.
 
-There are other modules like L<Data::Random>, L<App::Genpass>, L<String::MkPasswd>, L<Crypt::RandPasswd>, L<Crypt::GeneratePassword>, L<Data::SimplePassword>, L<Crypt::RandPasswd>, and L<String::MkPasswd> but they use insecure PRNGs such as C<rand()> or mersenne twister, don't adequately deal with bias, and/or don't let you specify generic alphabets.
+There are other modules like L<Data::Random>, L<App::Genpass>, L<String::MkPasswd>, L<Crypt::RandPasswd>, L<Crypt::GeneratePassword>, and L<Data::SimplePassword> but they use insecure PRNGs such as C<rand()> or mersenne twister, don't adequately deal with bias, and/or don't let you specify generic alphabets.
 
 L<Bytes::Random::Secure> has alphabets (aka "bags"), uses ISAAC, and avoids mod bias using the re-roll algorithm. It is much slower than Session::Token (even when using L<Math::Random::ISAAC::XS>) but does support alphabets larger than C<256> and might work in environments without XS.
 
 Neil Bowers has conducted a L<3rd party review|http://neilb.org/reviews/passwords.html> of various token/password generation modules including Session::Token.
 
+Leo Zovic has created a L<Common Lisp implementation of session-token|https://github.com/Inaimathi/session-token>.
+
 
 
 =head1 AUTHOR
